@@ -213,14 +213,6 @@ impl TlvDerive {
             ));
         }
 
-        if attr.args.contains_key(SKIP) {
-            return Err(Error::new(
-                Span::call_site(),
-                "presence of TLV attribute for the skipped field does not \
-                 make sense",
-            ));
-        }
-
         let mut tlv = if let Some(tlv) = attr
             .args
             .get(TLV)
@@ -236,6 +228,14 @@ impl TlvDerive {
             None
         };
 
+        if tlv.is_some() && attr.args.contains_key(SKIP) {
+            return Err(Error::new(
+                Span::call_site(),
+                "presence of TLV attribute for the skipped field does not \
+                 make sense",
+            ));
+        }
+
         if tlv.is_none() && is_global {
             tlv = Some(TlvDerive::None)
         }
@@ -248,7 +248,7 @@ impl TlvDerive {
         field: &Field,
         name: TokenStream2,
         fields: &mut Vec<TokenStream2>,
-        tlvs: &mut BTreeMap<usize, TokenStream2>,
+        tlvs: &mut BTreeMap<usize, (TokenStream2, bool)>,
         aggregator: &mut Option<TokenStream2>,
     ) -> Result<()> {
         match self {
@@ -257,15 +257,18 @@ impl TlvDerive {
                 Ok(())
             }
 
-            TlvDerive::Typed(type_no) => if let Type::Path(TypePath {
-                path,
-                ..
-            }) = &field.ty
-            {
-                if let Some(PathSegment { ident, .. }) = path.segments.last() {
-                    if *ident == ident!(Option) {
+            TlvDerive::Typed(type_no) => {
+                if let Type::Path(TypePath { path, .. }) = &field.ty {
+                    if let Some(PathSegment { ident, .. }) =
+                        path.segments.last()
+                    {
                         let n = name.to_string();
-                        if tlvs.insert(*type_no, name).is_some() {
+                        let entry = if *ident == ident!(Option) {
+                            (name, true)
+                        } else {
+                            (name, false)
+                        };
+                        if tlvs.insert(*type_no, entry).is_some() {
                             return Err(Error::new(
                                 field.span(),
                                 format!(
@@ -274,21 +277,11 @@ impl TlvDerive {
                                     type_no, n
                                 ),
                             ));
-                        } else {
-                            Ok(())
                         }
-                    } else {
-                        Err(())
                     }
-                } else {
-                    Err(())
                 }
-            } else {
-                Err(())
+                Ok(())
             }
-            .map_err(|_| {
-                Error::new(field.span(), "TLV fields must be optionals in type")
-            }),
 
             TlvDerive::Unknown => {
                 if let Type::Path(TypePath { path, .. }) = &field.ty {
